@@ -4,8 +4,9 @@ Extend this engine to give it more functionality.
 """
 import os
 import json
-import time
 from json.decoder import JSONDecodeError
+from typing import Union
+from urllib.parse import urljoin
 
 import requests
 from requests import Response
@@ -20,14 +21,17 @@ class BaseEngine:
     OCD_DTL_REQUESTS_PER_QUOTA_TIME = int(os.getenv('OCD_DTL_REQUESTS_PER_QUOTA_TIME', 5))
     logger.debug(f'Throttle selected: {OCD_DTL_REQUESTS_PER_QUOTA_TIME} queries per {OCD_DTL_QUOTA_TIME}s')
 
+    Json = Union[dict, list]  # json like object that can be a dict or root level array
+
     SET_MAX_RETRY = 3
 
-    def __init__(self, url: str, token_url: str, tokens: list):
-        self.url = url
-        self.token_url = token_url
+    def __init__(self, endpoint_config: dict, environment: str, tokens: list):
+        self.endpoint_config = endpoint_config
+        self.environment = environment
+        self.url = self._build_url(endpoint_config, environment)
         self.tokens = tokens
         self.terminal_size = self._get_size_terminal()
-        self.token_generator = TokenGenerator(token_url)
+        self.token_generator = TokenGenerator(endpoint_config, environment=environment)
         self.headers = None
 
         self.SET_MAX_RETRY = 3
@@ -57,8 +61,7 @@ class BaseEngine:
 
         if not headers.get('Authorization'):
             fresh_tokens = self.token_generator.get_token()
-            self.tokens = [f'Token {fresh_tokens["access_token"]}', f'Token {fresh_tokens["refresh_token"]}']
-            headers['Authorization'] = self.tokens[0]
+            self.replace_tokens(fresh_tokens)
         while True:
             response = self._send_request(url, method, headers, post_body)
             logger.debug(f'API response:\n{str(response.text)}')
@@ -67,6 +70,7 @@ class BaseEngine:
                 self._token_update(self._load_response(response))
             elif response.status_code == 422:
                 logger.warning('Bad authorization header. Updating token')
+                logger.debug(f'422 HTTP code: {response.text}')
                 self._token_update(self._load_response(response))
             elif response.status_code < 200 or response.status_code > 299:
                 logger.error(f'API returned non 2xx response code : {response.status_code}\n{response.text}'
@@ -138,22 +142,27 @@ class BaseEngine:
         """
         if dict_response.get('msg') == 'Missing Authorization Header':
             fresh_tokens = self.token_generator.get_token()
-            self.tokens = [f'Token {fresh_tokens["access_token"]}', f'Token {fresh_tokens["refresh_token"]}']
-            self.headers['Authorization'] = self.tokens[0]
+            self.replace_tokens(fresh_tokens)
             return False
         elif dict_response.get('msg') == 'Bad Authorization header. Expected value \'Token <JWT>\'':
             fresh_tokens = self.token_generator.get_token()
-            self.tokens = [f'Token {fresh_tokens["access_token"]}', f'Token {fresh_tokens["refresh_token"]}']
-            self.headers['Authorization'] = self.tokens[0]
+            self.replace_tokens(fresh_tokens)
             return False
         elif dict_response.get('msg') == 'Token has expired':
-            fresh_token = self.token_generator.refresh_token(self.tokens[1])
-            self.tokens = [f'Token {fresh_token["access_token"]}', self.tokens[1]]
-            self.headers['Authorization'] = self.tokens[0]
+            fresh_tokens = self.token_generator.refresh_token(self.tokens[1])
+            self.replace_tokens(fresh_tokens)
             return False
 
         return True
 
+    def replace_tokens(self, fresh_tokens: dict):
+        access_token = fresh_tokens["access_token"]
+        # Update of the refresh token is optional
+        refresh_token = fresh_tokens.get('refresh_token', self.tokens[1].replace('Token ', ''))
+
+        self.tokens = [f'Token {access_token}', f'Token {refresh_token}']
+        self.headers['Authorization'] = self.tokens[0]
+
     def _pretty_debug_request(self, url: str, method: str, data: dict, headers: dict, tokens: list):
         """
         Return pretty debug string
@@ -175,3 +184,12 @@ class BaseEngine:
                  f' - refresh_token: \n{tokens[1]}\n' +
                  '-' * self.terminal_size)
         return debug
+
+    def _build_url(self, endpoint_config: dict, environment: str):
+        """To be implemented by each subclass"""
+        raise NotImplemented()
+
+    def _build_url_for_endpoint(self, endpoint_name):
+        base_url = urljoin(self.endpoint_config['main'][self.environment], self.endpoint_config['api_version'])
+        enpoints = self.endpoint_config['endpoints']
+        return urljoin(base_url, enpoints[endpoint_name], allow_fragments=True)
