@@ -5,6 +5,7 @@ from datalake_scripts.common.base_script import BaseScripts
 from datalake_scripts.common.logger import logger
 from datalake_scripts.engines.get_engine import LookupThreats
 from datalake_scripts.engines.post_engine import PostEngine
+from datalake_scripts.helper_scripts.output_builder import CsvBuilder
 
 
 def output_type2header(v, parser):
@@ -86,9 +87,8 @@ def main(override_args=None):
     args.output_type = output_type2header(args.output_type, parser)
     hashkey_only = not args.threat_details
     # Load api_endpoints and tokens
-    endpoint_url, main_url, tokens = starter.load_config(args)
-    url_lookup_threats = main_url + endpoint_url['endpoints']['lookup']
-    get_engine_lookup_threats = LookupThreats(url_lookup_threats, main_url, tokens)
+    endpoint_config, main_url, tokens = starter.load_config(args)
+    get_engine_lookup_threats = LookupThreats(endpoint_config, args.env, tokens)
     list_threats = list(args.threats) if args.threats else []
     if args.input:
         if args.is_csv:
@@ -104,9 +104,11 @@ def main(override_args=None):
         list_threats,
         args.atom_type,
         hashkey_only,
-        args.output_type
     )
 
+    if args.output_type == 'text/csv':
+        response_dict = CsvBuilder.create_look_up_csv(response_dict, args.atom_type, has_details=args.threat_details)
+
     if args.output:
         starter.save_output(args.output, response_dict)
         logger.debug(f'Results saved in {args.output}\n')
